indexing

   description: "Abstract class that covers Posix semaphore routines."

   author: "Berend de Boer"
   date: "$Date: 2001/12/27 $"
   revision: "$Revision: #1 $"


-- deferred
class

   POSIX_SEMAPHORE


inherit

   POSIX_BASE

   PAPI_SEMAPHORE


feature -- commands

   attempt_acquire is
         -- Lock the semaphore only if it is not locked. If it is locked
         -- by some process, this command returns immediately and the
         -- semaphore is not locked
      require
         initialized: is_initialized
         unlocked: not is_locked
      local
         r: INTEGER
      do
         r := posix_sem_trywait (sem)
         is_locked := r = 0
         if not is_locked and r /= EAGAIN then
            raise_posix_error
         end
      end

   acquire is
         -- lock the semaphore
      require
         initialized: is_initialized
         unlocked: not is_locked
      do
         safe_call (posix_sem_wait (sem))
         is_locked := True
      end

   release is
         -- unlock the semaphore
      require
         locked: is_locked
         initialized: is_initialized
      do
         safe_call (posix_sem_post (sem))
         is_locked := False
      ensure
         unlocked: not is_locked
      end


feature -- queries

   is_initialized: BOOLEAN is
         -- True if semaphore is initialized/opened/created
      do
         Result := sem /= default_pointer
      end

   is_locked: BOOLEAN
         -- True if this process has locked the semaphore

   supports_semaphores: BOOLEAN is
         -- True if semaphores are supported
         -- most systems support unnamed semaphores, but still return False here
      do
         Result := posix_semaphores
      end

   value: INTEGER is
         -- value of semaphore if not locked.
         -- Value is <= 0 if this semaphore is locked.
      do
         safe_call (posix_sem_getvalue (sem, sem_value.ptr))
         Result := sem_value.peek_integer (0)
      end


feature {NONE} -- state

   my_sem_value: STDC_BUFFER

   sem_value: STDC_BUFFER is
         -- buffer that can hold result of posix_sem_getvalue
      do
         if my_sem_value = Void then
            create my_sem_value.allocate_and_clear (Integer_bits // 8)
         end
         Result := my_sem_value
      end

   sem: POINTER


invariant

   sem_value_valid: sem_value /= Void


end -- class POSIX_SEMAPHORE
