indexing

   description: "Class that can execute something that should take %
   %X seconds to complete. If it takes more than X seconds, the command is%
   %terminated and execute returns False."

   idea: "Inherit from this class and implement do_execute."

   author: "Berend de Boer"
   date: "$Date: 2001/12/27 $";
   revision: "$Revision: #1 $"


deferred class

   POSIX_TIMED_COMMAND


inherit

   POSIX_BASE

   LAST_SIGNAL

   PAPI_UNISTD


feature -- creation

   make (a_seconds: INTEGER) is
      require
         valid_seconds: a_seconds >= 1 and a_seconds <= 65535
      do
         seconds := a_seconds
      end


feature -- execution

   execute: BOOLEAN is
         -- Return true if `do_execute' completed within the time it
         -- should execute.
      require
         -- the signal SIGNAL_ALARM causes an Eiffel exception
      local
         signalled: BOOLEAN
         execute_tried: BOOLEAN
         done: BOOLEAN
      do
         if execute_tried then
            Result := not signalled
         else
            execute_tried := True
            done := False
            alarm_on
            do_execute
            done := True
            alarm_off
            Result := True
         end
      rescue
         signalled :=
            (not done) and
            (is_signal and then signal = SIGNAL_ALARM)
         retry
      end


feature {NONE}

   alarm_on is
         -- Turns on alarm request at `seconds' realseconds.  You can
         -- check `remaining_seconds' > 0 if a previous alarm was
         -- scheduled.
      do
         remaining_seconds := posix_alarm (seconds)
      end

   alarm_off is
         -- Turns off alarm request
         -- remaining seconds are in `remaining_seconds'
      do
         remaining_seconds := posix_alarm (0)
      end

   do_execute is
         -- does the real work
      deferred
      end


feature -- state

   remaining_seconds: INTEGER
         -- number of seconds left in previous request

   seconds: INTEGER
         -- the number of seconds available to execute the command

   set_seconds (a_seconds: INTEGER) is
      do
         seconds := a_seconds
      end


invariant

   valid_seconds: seconds >= 1

end -- class POSIX_TIMED_COMMAND
