indexing

   description: "The exit handler that calls all other exit handlers."

   author: "Berend de Boer"
   date: "$Date: 2001/12/27 $"
   revision: "$Revision: #1 $"


class 

   STDC_EXIT_SWITCH
   
inherit

   EPX_SINGLETON
   
   MEMORY
      redefine
         dispose
      end

   
creation
   
   make

   
feature -- creation
   
   make is
      do
         create exit_handlers.make (4)
         epx_set_exit_switch (Current)
      end
   
   
feature -- gc
   
   dispose is
         -- make sure remaining handlers are made harmless
      do
         epx_clear_exit_switch
         precursor
      end
   

feature -- install/uninstall handlers
   
   is_installed (handler: STDC_EXIT_HANDLER): BOOLEAN is
      do
         Result := exit_handlers.exists_on_stack (handler)
      end

   install (handler: STDC_EXIT_HANDLER) is
      require
         valid_handler: handler /= Void
         handler_not_installed: not is_installed (handler)
      do
         exit_handlers.push (handler)
      ensure
         handler_installed: is_installed (handler)
      end
   
   uninstall (handler: STDC_EXIT_HANDLER) is
      require
         valid_handler: handler /= Void
         handler_installed: is_installed (handler)
      do
         exit_handlers.remove (handler)
      ensure
         handler_not_installed: not is_installed (handler)
      end


feature {NONE} -- core switch
   
   at_exit is
         -- core routine, is called when program terminates
      do         
         from
         until
            exit_handlers.is_empty
         loop
            exit_handlers.top.execute
            exit_handlers.pop
         end
      end
   

feature {NONE} -- state
   
   exit_handlers: EPX_STACK [STDC_EXIT_HANDLER]
   
   
feature {NONE} -- C binding
   
   epx_clear_exit_switch is
      external "C"
      end
   
   epx_set_exit_switch (a_switch: STDC_EXIT_SWITCH) is
      require
         valid_switch: a_switch /= Void
      external "C"
      end
   

feature {NONE} -- private   
   
   frozen the_singleton: EPX_SINGLETON is
      once
         Result := Current
      end


end
