indexing

   description: "String utilities such as converting strings%
   %to pointers and vice versa."

   thanks: "The mico/E team for the idea."

   date: "$Date: 2001/12/27 $"
   revision: "$Revision: #1 $"

deferred class

   ABSTRACT_STRING_HELPER


feature -- contracts for platform specific STRING_HELPER

   string_to_pointer (s: STRING): POINTER is
      require
         -- any `s', including Void
      deferred
      ensure
         void_result: s = Void implies Result = default_pointer
      end

   pointer_to_string (p: POINTER): STRING is
         -- returns a new string
      require
         -- any `p' including default_pointer
      deferred
      ensure
         empty_string: p = default_pointer implies equal(Result, "")
      end

   set_string_from_pointer (s: STRING; p: POINTER) is
         -- copies contents of p to s
      require
         valid_s: s /= Void
      deferred
      ensure
         empty_string: p = default_pointer implies s.count = 0
      end


feature -- string utilities that have different names per compiler...

   clear (s: STRING) is
      deferred
      ensure
         really_empty: s.count = 0
      end

   insert_character (s: STRING; c: CHARACTER; i: INTEGER) is
      deferred
      end

   make_with_capacity (capacity: INTEGER): STRING is
         -- create a new string
      require
         valid_capacity: capacity >= 0
      deferred
      ensure
         have_result: Result /= Void
      end


feature -- general string utilities

   chop (s: STRING) is
         -- remove newline character (if any) from `s'
         -- only when the last characters from last_string are %N or
         -- %R%N, they're removed.
      do
         if s.count > 0 and then s.item (s.count) = '%N' then
            s.remove (s.count)
            if s.count > 0 and then s.item (s.count) = '%R' then
               s.remove (s.count)
            end
         end
      end

   has_substring (s, substring: STRING): BOOLEAN is
         -- True if `s' contains the string `substring'
      do
         Result := s.substring_index (substring, 1) /= 0
      end

   make_spaces (count: INTEGER): STRING is
         -- return string with `count' spaces
      require
         valid_count: count >= 0
      local
         i: INTEGER
      do
         create Result.make (count)
         clear (Result)
         from
            i := 1
         until
            i > count
         loop
            Result.append_character (' ')
            i := i + 1
         end
      ensure
         enough_spaces: Result.count = count
      end

   split_on(s: STRING; on: CHARACTER): ARRAY[STRING] is
         -- split a string on a given character.
      local
         p, start: INTEGER
         tmp_string: STRING
         count: INTEGER
      do
         create Result.make (0, 32)

         if s.count > 0 then
            from
               start := 1
               p := s.index_of (on, start)
            until
               p = 0
            loop
               if Result.upper = count then
                  Result.resize (0, 2*count)
               end
               tmp_string := s.substring (start, p-1)
               Result.put (tmp_string, count)
               count := count + 1

               start := p + 1
               p := s.index_of (on, start)
            end

            if start <= s.count then
               if Result.upper = count then
                  Result.resize (0, 2*count)
               end
               tmp_string := s.substring (start, s.count)
               Result.put (tmp_string, count)
               count := count + 1
            end
         end

         Result.resize (0, count-1)
      end

   trim (s: STRING): STRING is
         -- remove leading and trailing white space
         -- whitespace is blanks and tabs
      local
         i: INTEGER
         whitespace: BOOLEAN
         c: CHARACTER
      do
         create Result.make_from_string (s)
         from
            i := 1
         until
            i = Result.count
         loop
            c := Result.item (i)
            whitespace := c = ' ' or else c = '%T'
            if whitespace then
               Result.remove (i)
            else
               i := i + 1
            end
         end
      end


end -- class ABSTRACT_STRING_HELPER
